/* ////////////////////////////////////////////////////////////

Polygon Crest
Copyright (c) 2015 Soji Yamakawa.  All rights reserved.
http://www.ysflight.com

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
   this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, 
   this list of conditions and the following disclaimer in the documentation 
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

File Name: ysshellext_geomcalc.h
//////////////////////////////////////////////////////////// */

#ifndef YSSHELLEXT_GEOMCALC_IS_INCLUDED
#define YSSHELLEXT_GEOMCALC_IS_INCLUDED
/* { */

#include <ysshellext.h>

/*! Calculate dihedral angle between two polygons. 
    These functions calculate normals based on vertex locations, not cached normal vectors. */
const double YsShellExt_CalculateDihedralAngle(const class YsShellExt &shl,const YsShellPolygonHandle plHd[2]);
const double YsShellExt_CalculateDihedralAngle(const class YsShellExt &shl,YsShellPolygonHandle plHd0,YsShellPolygonHandle plHd1);

/*! Calculate dihedral angle across an edge.
    If the edge is not used by two polygons, this function returns 0.0 */
const double YsShellExt_CalculateDihedralAngle(const class YsShellExt &shl,const YsShellVertexHandle edVtHd[2]);
const double YsShellExt_CalculateDihedralAngle(const class YsShellExt &shl,const YsShellVertexHandle edVtHd0,const YsShellVertexHandle edVtHd1);
const double YsShellExt_CalculateDihedralAngle(const class YsShellExt &shl,const YsShellPolygonHandle plHd,int nThEdgeZeroBased);

/*! Calculate total dihedral angle of edges:
    This function internally uses YsShellExt_CalculateDihedralAngle function.
           edVtHd[0]
    +--(1)--+
    |      /|
    |     / |
   (2)   /  |
    |  (5)  |
    |  /   (4)
    | /     |
    |/      |
    +--(3)--+
  edVtHd[1] */
const double YsShellExt_CalculateTotalDihedralAngleAroundEdge(const class YsShellExt &shl,const YsShellVertexHandle edVtHd[2]);
const double YsShellExt_CalculateTotalDihedralAngleAroundEdge(const class YsShellExt &shl,const YsShellVertexHandle edVtHd0,const YsShellVertexHandle edVtHd1);

/*! This function calculates average normal of the given polygons.  It does not use cached normal.
    It only rely on vertex positions. 
    It is not orientation sensitive.  The returned normal may be reverse of the normal that can be calculated based on the polygon orientation. */
const YsVec3 YsShell_CalculatePolygonAverageNormal(const YsShell &shl,YSSIZE_T nPl,const YsShellPolygonHandle plHdArray[]);
template <const int N>
const YsVec3 YsShell_CalculatePolygonAverageNormal(const YsShell &shl,const YsArray <YsShellPolygonHandle,N> &plHdArray)
{
	return YsShell_CalculatePolygonAverageNormal(shl,plHdArray.GetN(),plHdArray);
}


class YsShellExt_NormalCalculator
{
public:
	enum VERTEX_NORMAL_CALCULATION_WEIGHT
	{
		NO_WEIGHT
	};

	VERTEX_NORMAL_CALCULATION_WEIGHT weight;
	YsShellExt::FaceGroupHandle limitFgHd;

	YsShellExt_NormalCalculator();
	YsVec3 CalculateVertexNormal(const YsShellExt &shl,YsShellVertexHandle vtHd) const;
	YsVec3 CalculateEdgeNormal(const YsShellExt &shl,const YsShellVertexHandle edVtHd[2]) const;
};


/*! See Garland and Heckbert 1997. 
    The following calculation will give total of square distances from the planes converging at the vertex.

                (x)
     [x y z 1]Kp(y)
                (z)
                (1)

*/
void YsShellExt_CalculateQuadricErrorMetric(YsMatrix4x4 &Kp,const YsShellExt &shl,YsShellVertexHandle vtHd);


/*! Calculate error from the quadric error metric. 
    It measures an error if the vertex used for calculating Kp is moved to the given location. */
double YsShellExt_CalculateQuadricError(const YsMatrix4x4 &Kp,const YsVec3 &pos);


/*! Calculate total angle taken by polygons around the vertex. */
double YsShellExt_CalculateTotalFaceAngle(const YsShellExt &shl,YsShellVertexHandle vtHd);

/*! Calculate total fan angle of the polygon at vertex. */
double YsShellExt_CalculatePolygonFaceAngle(const YsShellExt &shl,YsShellPolygonHandle plHd,YsShellVertexHandle vtHd);

/*! Calculate total fan angle of the polygon defined by plVtHd[0]...plVtHd[nVt-1] at vertex. */
double YsShellExt_CalculatePolygonFaceAngle(const YsShellExt &shl,YSSIZE_T nVt,const YsShellVertexHandle plVtHd[],YsShellVertexHandle vtHd);

/*! Calculate total fan angle of the polygon defined by plVtHd[0]...plVtHd[nVt-1] at vertex. */
template <const int N>
double YsShellExt_CalculatePolygonFaceAngle(const YsShellExt &shl,const YsArray <YsShellVertexHandle,N> &plVtHd,YsShellVertexHandle vtHd)
{
	return YsShellExt_CalculatePolygonFaceAngle(shl,plVtHd.GetN(),plVtHd,vtHd);
}

YSBOOL YsShellExt_IsFaceGroupPlanar(const YsShellExt &shl,YsShellExt::FaceGroupHandle fgHd,const double cosineTolerance=1.0-YsTolerance);

/*! Returns the longest edge of the polygon and the height measured from the longest edge. */
void YsShellExt_CalculateAspectRatio(double &length,double &height,YSSIZE_T np,const YsVec3 p[]);

/*! Returns the longest edge of the polygon and the height measured from the longest edge. */
template <const int N>
void YsShellExt_CalculateAspectRatio(double &length,double &height,const YsArray <YsVec3,N> &plg)
{
	YsShellExt_CalculateAspectRatio(length,height,plg.GetN(),plg);
}

/*! Returns the longest edge of the polygon and the height measured from the longest edge. */
void YsShellExt_CalculateAspectRatio(double &length,double &height,const YsShell &shl,YsShellPolygonHandle plHd);

/*! Returns the longest edge of the polygon and the height measured from the longest edge. */
void YsShellExt_CalculateAspectRatio(double &length,double &height,const YsShell &shl,YSSIZE_T nv,const YsShellVertexHandle v[]);

/*! Returns the longest edge of the polygon and the height measured from the longest edge. */
template <const int N>
void YsShellExt_CalculateAspectRatio(double &length,double &height,const YsShell &shl,const YsArray <YsShellVertexHandle,N> &v)
{
	YsShellExt_CalculateAspectRatio(length,height,shl,v.GetN(),v);
}


// class YsShellExt_PolygonProjection has been moved to ysshellext_projectionutil.h

/* } */
#endif
