/* ////////////////////////////////////////////////////////////

Polygon Crest
Copyright (c) 2015 Soji Yamakawa.  All rights reserved.
http://www.ysflight.com

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
   this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, 
   this list of conditions and the following disclaimer in the documentation 
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

File Name: ysshellext_sewingutil.h
//////////////////////////////////////////////////////////// */

#ifndef YSSHELLEXT_SEWINGUTIL_IS_INCLUDED
#define YSSHELLEXT_SEWINGUTIL_IS_INCLUDED
/* { */

#include <ysshellext.h>
#include "ysshellext_localop.h"

/*! This class creates an information needed for sewing from one point on the shell to another. 
    The application first creates a vertex sequence and vertex-insertion info by calling MakeInfo. 
    Then iterate through all of AllVtxOnEdge() and insert vertices on the edges.
    Then call MakePolygonSplitInfo, which makes information of how the polygons must split.
    Finally, polygon-split info can be applied to finalize it.

	Example:
				YsShellExtEdit &shl;  <= input

				YsShellExtEdit::StopIncUndo undo(shl);

				YsShellExt_SewingInfo sewingInfo;

				sewingInfo.MakeInfo(shl,fromVtHd,toVtHd);
				for(auto &vone : sewingInfo.AllVtxOnEdge())
				{
					vone.createdVtHd=shl.AddVertex(vone.pos);

					YsShellExtEdit_TopologyUtil topoUtil;
					topoUtil.InsertVertexOnEdge(shl,vone.edVtHd,vone.createdVtHd);
				}

				sewingInfo.MakePolygonSplitInfo(shl.Conv());
				for(auto &ps : sewingInfo.AllPolygonSplit())
				{
					YsShellExtEdit_ApplyPolygonSplit(shl,ps);
				}
*/
class YsShellExt_SewingInfo : YsShell_LocalOperation
{
protected:
	YsArray <YsShell_LocalOperation::EdgeSplit> vtxOnEdge;
	YsArray <YsShell::VertexPolygonAndIndex> vtxSequence;

	YsArray <YsShell_LocalOperation::PolygonSplit> plgSplit;

public:
	class AllVtxOnEdgeMask
	{
	friend class YsShellExt_SewingInfo;
	private:
		YsArray <EdgeSplit> *vtxOnEdge;
	public:
		inline EdgeSplit &operator[](YSSIZE_T idx)
		{
			return (*vtxOnEdge)[idx];
		}
		inline YSSIZE_T GetN(void) const
		{
			return vtxOnEdge->GetN();
		}

		typedef YsIndexIterator <AllVtxOnEdgeMask,EdgeSplit> iterator;
		inline iterator begin()
		{
			iterator iter(this);
			return iter.begin();
		}
		inline iterator end()
		{
			iterator iter(this);
			return iter.end();
		}
	};

	class AllPolygonSplitMask
	{
	friend class YsShellExt_SewingInfo;
	private:
		const YsArray <YsShell_LocalOperation::PolygonSplit> *plgSplitPtr;
	public:
		inline const PolygonSplit &operator[](YSSIZE_T idx) const
		{
			return (*plgSplitPtr)[idx];
		}
		inline YSSIZE_T GetN(void) const
		{
			return plgSplitPtr->GetN();
		}

		typedef YsIndexIterator <const AllPolygonSplitMask,const YsShell_LocalOperation::PolygonSplit> iterator;
		inline iterator begin()
		{
			iterator iter(this);
			return iter.begin();
		}
		inline iterator end()
		{
			iterator iter(this);
			return iter.end();
		}
	};

public:
	void CleanUp(void);
	YSRESULT MakeInfo(const YsShellExt &shl,YsShellVertexHandle vtHdFrom,YsShellVertexHandle vtHdTo);

	AllVtxOnEdgeMask AllVtxOnEdge(void);

	/*! Call this function after populating vtxOnEdge. */
	void MakePolygonSplitInfo(const YsShellExt &shl);

	AllPolygonSplitMask AllPolygonSplit(void) const;
};

inline YsShellExt_SewingInfo::AllVtxOnEdgeMask::iterator begin(YsShellExt_SewingInfo::AllVtxOnEdgeMask &dat)
{
	return dat.begin();
}
inline YsShellExt_SewingInfo::AllVtxOnEdgeMask::iterator end(YsShellExt_SewingInfo::AllVtxOnEdgeMask &dat)
{
	return dat.end();
}

inline YsShellExt_SewingInfo::AllPolygonSplitMask::iterator begin(YsShellExt_SewingInfo::AllPolygonSplitMask &dat)
{
	return dat.begin();
}
inline YsShellExt_SewingInfo::AllPolygonSplitMask::iterator end(YsShellExt_SewingInfo::AllPolygonSplitMask &dat)
{
	return dat.end();
}


/* } */
#endif
